import { HttpErrorResponse, HttpStatusCode } from '@angular/common/http';
import { TestBed } from '@angular/core/testing';
import { COMMAND_ERROR_MESSAGES, CommandErrorMessage, CommandOrder, CommandResource } from '@alfa-client/command-shared';
import { NavigationService } from '@alfa-client/navigation-shared';
import { ApiError } from '@alfa-client/tech-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { SnackBarService } from '@alfa-client/ui';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { provideMockActions } from '@ngrx/effects/testing';
import { Action } from '@ngrx/store';
import { provideMockStore } from '@ngrx/store/testing';
import { cold, hot } from 'jest-marbles';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createApiError, createHttpErrorResponse } from 'libs/tech-shared/test/error';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { Observable, of } from 'rxjs';
import { createLoeschAnforderungResource } from '../../../test/loesch-anforderung';
import { LoeschAnforderungResource } from '../loesch-anforderung.model';
import { LoeschAnforderungEffects } from './loesch-anforderung.effects';
import { LoeschAnforderungRepository } from './loesch-anforderung.repository';

import faker from '@faker-js/faker';

import * as CommandActions from '../../../../command-shared/src/lib/+state/command.actions';
import * as VorgangActions from '../../../../vorgang-shared/src/lib/+state/vorgang.actions';
import * as LoeschAnforderungActions from './loesch-anforderung.actions';

describe('LoeschAnforderungEffects', () => {
	let actions: Observable<Action>;
	let effects: LoeschAnforderungEffects;

	const repository: Mock<LoeschAnforderungRepository> = mock(LoeschAnforderungRepository);
	const snackbarService: Mock<SnackBarService> = mock(SnackBarService);
	const navigationService: Mock<NavigationService> = mock(NavigationService);

	beforeEach(() => {
		TestBed.configureTestingModule({
			providers: [
				LoeschAnforderungEffects,
				provideMockActions(() => actions),
				provideMockStore(),
				{
					provide: LoeschAnforderungRepository,
					useValue: repository
				},
				{
					provide: SnackBarService,
					useValue: snackbarService
				},
				{
					provide: NavigationService,
					useValue: navigationService
				}
			]
		});

		effects = TestBed.inject(LoeschAnforderungEffects);
	});

	describe('loadLoeschAnforderung', () => {

		const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.LOESCH_ANFORDERUNG]);
		const action = VorgangActions.loadVorgangWithEingangSuccess({ vorgangWithEingang });

		it('should call repository', () => {
			actions = of(action);

			effects.loadLoeschAnforderung$.subscribe();

			expect(repository.getLoeschAnforderung).toHaveBeenCalledWith(vorgangWithEingang);
		})

		it('should dispatch loadLoeschAnforderungSuccess action', () => {
			const loeschAnforderung: LoeschAnforderungResource = createLoeschAnforderungResource();
			repository.getLoeschAnforderung.mockReturnValue(of(loeschAnforderung));

			actions = hot('-a-|', { a: action });

			const expected = hot('-a-|', { a: LoeschAnforderungActions.loadLoeschAnforderungSuccess({ loeschAnforderung }) });
			expect(effects.loadLoeschAnforderung$).toBeObservable(expected);
		})

		it('should dispatch loadLoeschAnforderungFailure action', () => {
			const apiError: ApiError = createApiError()
			const error = { error: { error: apiError } };
			const errorResponse = cold('-#', {}, error);
			repository.getLoeschAnforderung = jest.fn(() => errorResponse);

			const expected = cold('--b', { b: LoeschAnforderungActions.loadLoeschAnforderungFailure({ apiError }) });
			actions = hot('-a', { a: action });

			expect(effects.loadLoeschAnforderung$).toBeObservable(expected);
		})
	})

	describe('showVorgangLoeschenSuccessSnackbar', () => {

		const command: CommandResource = { ...createCommandResource(), order: CommandOrder.VORGANG_LOESCHEN };
		const action = CommandActions.createCommandSuccess({ command });

		it('should call sncakbarService on vorgangLoeschen command', () => {
			actions = of(action)

			effects.showVorgangLoeschenSuccessSnackbar$.subscribe();

			expect(snackbarService.show).toHaveBeenCalled();
		})
	})

	describe('loeschAnforderungZuruecknehmenFailedAndReloadVorgangSuccess', () => {

		const action = LoeschAnforderungActions.loeschAnforderungZuruecknehmenFailedAndReloadVorgangSuccess();

		it('should show snackbar', () => {
			actions = of(action)

			effects.loeschAnforderungZuruecknehmenFailedAndReloadVorgangSuccess$.subscribe();

			expect(snackbarService.showError).toHaveBeenCalledWith(COMMAND_ERROR_MESSAGES[CommandErrorMessage.CONCURRENT_MODIFICATION]);
		})
	})

	describe('loeschAnforderungZuruecknehmenFailedAndReloadVorgangFailure', () => {

		const error: HttpErrorResponse = { ...createHttpErrorResponse(), status: HttpStatusCode.NotFound};
		const actionError = <HttpErrorResponse>{ error };
		const overviewUrl: string = faker.internet.url();
		const action = LoeschAnforderungActions.loeschAnforderungZuruecknehmenFailedAndReloadVorgangFailure({ error: actionError, overviewUrl });

		it('should show snackbar', () => {
			actions = of(action)

			effects.loeschAnforderungZuruecknehmenFailedAndReloadVorgangFailure$.subscribe();

			expect(snackbarService.showInfo).toHaveBeenCalledWith('Der Vorgang wurde zwischenzeitlich gelöscht.');
		})

		it('should navigate to overview', () => {
			actions = of(action)

			effects.loeschAnforderungZuruecknehmenFailedAndReloadVorgangFailure$.subscribe();

			expect(navigationService.navigate).toHaveBeenCalledWith(overviewUrl);
		})
	})
});