/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { EMPTY_STRING } from '@alfa-client/tech-shared';
import { createUserProfileResource } from '../../../user-profile-shared/test/user-profile';
import { UserProfileResource } from './user-profile.model';
import { existsName, getUserName, getUserNameInitials, NO_NAME_MESSAGE, UNKNOWN_USER } from './user-profile.util';

describe('UserProfileUtil', () => {

	describe('existsName', () => {

		it.each([null, EMPTY_STRING])('should return true if only firstName exists and lastName has value %s', (value: null | string) => {
			const exists: boolean = existsName({ ...createUserProfileResource(), lastName: value });

			expect(exists).toBeTruthy();
		})

		it.each([null, EMPTY_STRING])('should return true if only lastName exists and firstName has value %s', (value: null | string) => {
			const exists: boolean = existsName({ ...createUserProfileResource(), firstName: value });

			expect(exists).toBeTruthy();
		})

		it.each([null, EMPTY_STRING])('should return false if either firstName nor lastName exists', (value: null | string) => {
			const exists: boolean = existsName({ ...createUserProfileResource(), firstName: value, lastName: value });

			expect(exists).toBeFalsy();
		})
	})

	describe('get user name', () => {

		it('should return unknown user message on null', () => {
			const userName: string = getUserName(null);

			expect(userName).toBe(UNKNOWN_USER);
		})

		it('should return full name', () => {
			const userProfile: UserProfileResource = createUserProfileResource();

			const userName: string = getUserName(userProfile);

			expect(userName).toBe(`${userProfile.firstName} ${userProfile.lastName}`);
		})

		it('should return if only firstName exists', () => {
			const userProfile: UserProfileResource = { ...createUserProfileResource(), lastName: null };

			const userName: string = getUserName(userProfile);

			expect(userName).toBe(userProfile.firstName);
		})

		it('should return only lastName exists', () => {
			const userProfile: UserProfileResource = { ...createUserProfileResource(), firstName: null };

			const userName: string = getUserName(userProfile);

			expect(userName).toBe(userProfile.lastName);
		})

		it('should return no name message if either firstName nor lastName exists', () => {
			const userProfile: UserProfileResource = { ...createUserProfileResource(), firstName: null, lastName: null };

			const userName: string = getUserName(userProfile);

			expect(userName).toBe(NO_NAME_MESSAGE);
		})
	})

	describe('get user name initials', () => {

		it('should return for full name', () => {
			const userProfile: UserProfileResource = createUserProfileResource();

			const userName: string = getUserNameInitials(userProfile);

			expect(userName).toBe(`${userProfile.firstName.substring(0, 1).toLocaleUpperCase()}${userProfile.lastName.substring(0, 1).toLocaleUpperCase()}`);
		})

		it('should return if only firstName exists', () => {
			const userProfile: UserProfileResource = { ...createUserProfileResource(), lastName: EMPTY_STRING };

			const userName: string = getUserNameInitials(userProfile);

			expect(userName).toBe(`${userProfile.firstName.substring(0, 1).toLocaleUpperCase()}`);
		})

		it('should return only lastName exists', () => {
			const userProfile: UserProfileResource = { ...createUserProfileResource(), firstName: EMPTY_STRING };

			const userName: string = getUserNameInitials(userProfile);

			expect(userName).toBe(`${userProfile.lastName.substring(0, 1).toLocaleUpperCase()}`);
		})

		it('should return empty string if either firstName nor lastName exists', () => {
			const userProfile: UserProfileResource = { ...createUserProfileResource(), firstName: EMPTY_STRING, lastName: EMPTY_STRING };

			const userName: string = getUserNameInitials(userProfile);

			expect(userName).toBe(EMPTY_STRING);
		})
	})
})