/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatTabsModule } from '@angular/material/tabs';
import { NoopAnimationsModule } from '@angular/platform-browser/animations';
import { HistorieContainerComponent } from '@alfa-client/historie';
import { HasLinkPipe, NotHasLinkPipe } from '@alfa-client/tech-shared';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { ExpansionPanelComponent } from '@alfa-client/ui';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { VorgangDetailAntragDataComponent } from './vorgang-detail-antrag-data/vorgang-detail-antrag-data.component';
import { VorgangDetailAttachmentListComponent } from './vorgang-detail-attachment-list/vorgang-detail-attachment-list.component';
import { VorgangDetailFormularDatenComponent } from './vorgang-detail-formular-daten.component';
import { VorgangDetailMetaDataComponent } from './vorgang-detail-meta-data/vorgang-detail-meta-data.component';
import { VorgangDetailRepresentationListComponent } from './vorgang-detail-representation-list/vorgang-detail-representation-list.component';

describe('VorgangDetailFormularDatenComponent', () => {
	let component: VorgangDetailFormularDatenComponent;
	let fixture: ComponentFixture<VorgangDetailFormularDatenComponent>;

	const tabMetaData: string = getTabByIndex(2);;
	const tabRepresentations: string = getTabByIndex(3);
	const tabAttachments: string = getTabByIndex(4);
	const tabHistorie: string = getTabByIndex(5);

	function getTabByIndex(index: number): string {
		return `div[role=tab]:nth-child(${ index })`
	}

	const disabledTabClass: string = 'mat-mdc-tab-disabled';

	const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();
	const vorgangWithHistorie: VorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.HISTORIE]);

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			imports: [
				NoopAnimationsModule,
				MatTabsModule
			],
			declarations: [
				HasLinkPipe,
				NotHasLinkPipe,
				VorgangDetailFormularDatenComponent,
				MockComponent(VorgangDetailAntragDataComponent),
				MockComponent(VorgangDetailMetaDataComponent),
				MockComponent(VorgangDetailAttachmentListComponent),
				MockComponent(VorgangDetailRepresentationListComponent),
				MockComponent(ExpansionPanelComponent),
				MockComponent(HistorieContainerComponent),
			]
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangDetailFormularDatenComponent);
		component = fixture.componentInstance;
		component.vorgangWithEingang = vorgangWithEingang;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('Metadaten', () => {

		it('should show tab enabled if Vorgang contains metadata', () => {
			component.metaData = { 'test': 'testValue' };

			fixture.detectChanges();
			const metaDataTab = getElementFromFixture(fixture, tabMetaData);

			expect(metaDataTab).not.toHaveClass(disabledTabClass);
		})

		it('should disable tab if Vorgang contains no metadata', () => {
			fixture.detectChanges();
			const metaDataTab = getElementFromFixture(fixture, tabMetaData);

			expect(metaDataTab).toHaveClass(disabledTabClass);
		})

		it('should show MetaData label', () => {
			fixture.detectChanges();
			const metaDataTab = getElementFromFixture(fixture, tabMetaData);

			expect(metaDataTab).toHaveTextContent('Metadaten');
		})
	})

	describe('Tab Datenrepräsentation', () => {

		const vorgangWithRepresentations: VorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.REPRESENTATIONS]);

		it('should be enabled and selected if representations links is present', () => {
			component.vorgangWithEingang = vorgangWithRepresentations;

			fixture.detectChanges();
			const tab = getElementFromFixture(fixture, tabRepresentations);

			expect(tab).not.toHaveClass(disabledTabClass);
			expect(tab).toHaveAttribute('aria-selected', 'true');
		})

		it('should be disable and not selected if representations link is NOT present', () => {
			component.vorgangWithEingang = vorgangWithEingang;

			fixture.detectChanges();
			const tab = getElementFromFixture(fixture, tabRepresentations);

			expect(tab).toHaveClass(disabledTabClass);
			expect(tab).toHaveAttribute('aria-selected', 'false');
		})

		it('should show number of Representations in the label', () => {
			component.vorgangWithEingang = vorgangWithRepresentations;

			fixture.detectChanges();
			const tab = getElementFromFixture(fixture, tabRepresentations);
			const tabLabel = `Datenrepräsentation (${vorgangWithRepresentations.eingang.numberOfRepresentations})`;

			expect(tab).toHaveTextContent(tabLabel);
		})

	})

	describe('Tab Anhänge', () => {

		const vorgangWithAttachments: VorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.ATTACHMENTS]);

		it('should be enabled if attachments link is present', () => {
			component.vorgangWithEingang = vorgangWithAttachments;

			fixture.detectChanges();
			const tab = getElementFromFixture(fixture, tabAttachments);

			expect(tab).not.toHaveClass(disabledTabClass);
		})

		it('should be disable if attachments link is NOT present', () => {
			component.vorgangWithEingang = vorgangWithEingang;

			fixture.detectChanges();
			const tab = getElementFromFixture(fixture, tabAttachments);

			expect(tab).toHaveClass(disabledTabClass);
		})

		it('should show number of Attachments in the label', () => {
			component.vorgangWithEingang = vorgangWithAttachments;

			fixture.detectChanges();
			const tab = getElementFromFixture(fixture, tabAttachments);
			const tabLabel = `Anhänge (${vorgangWithAttachments.eingang.numberOfAttachments})`;

			expect(tab).toHaveTextContent(tabLabel);
		})
	})

	describe('Tab Historie', () => {

		it('should be visible if Link "historie" exists', () => {
			component.vorgangWithEingang = vorgangWithHistorie;

			fixture.detectChanges();
			const tab = getElementFromFixture(fixture, tabHistorie);

			expect(tab).toBeInTheDocument();
		})

		it('should not be visible if no Link "historie"', () => {
			component.vorgangWithEingang = vorgangWithEingang;

			fixture.detectChanges();
			const tab = getElementFromFixture(fixture, tabHistorie);

			expect(tab).not.toBeInTheDocument();
		})

		it('should show historie label', () => {
			component.vorgangWithEingang = vorgangWithHistorie;

			fixture.detectChanges();
			const metaDataTab = getElementFromFixture(fixture, tabHistorie);

			expect(metaDataTab).toHaveTextContent('Historie');
		})
	})

});