/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import static de.ozgcloud.alfa.common.command.CommandController.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.nio.charset.StandardCharsets;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.itvsh.kop.common.test.TestUtils;
import de.ozgcloud.alfa.common.command.CommandController.CommandByRelationController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

@DisplayName("Command by relation controller")
class CommandByRelationControllerTest {

	@Spy
	@InjectMocks // NOSONAR
	private CommandByRelationController controller;
	@Mock
	private CommandService service;

	private MockMvc mockMvc;

	@BeforeEach
	void initMockMvc() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	@DisplayName("create Command by Vorgang")
	class CreateCommandByVorgang {

		@Captor
		private ArgumentCaptor<CreateCommand> createCommandCaptor;

		@BeforeEach
		void initTest() {
			when(service.createCommand(any(CreateCommand.class), anyLong())).thenReturn(CommandTestFactory.create());
		}

		@DisplayName("should return CREATED")
		@Test
		void returnCreated() throws Exception {
			doRequest().andExpect(status().isCreated());
		}

		@DisplayName("should call service")
		@Test
		void callService() throws Exception {
			doRequest();

			verify(service).createCommand(createCommandCaptor.capture(), eq(CommandTestFactory.RELATION_VERSION));
			assertThat(createCommandCaptor.getValue().getOrder()).isEqualTo(CommandTestFactory.ORDER);
			assertThat(createCommandCaptor.getValue().getVorgangId()).isEqualTo(CommandTestFactory.VORGANG_ID);
			assertThat(createCommandCaptor.getValue().getRelationId()).isEqualTo(CommandTestFactory.RELATION_ID);
			assertThat(createCommandCaptor.getValue().getRelationVersion()).isZero();
		}

		@DisplayName("should have location header")
		@Test
		void locationHeader() throws Exception {
			doRequest()
					.andExpect(header().stringValues("location", "http://localhost" + COMMANDS_PATH + "/" + CommandTestFactory.ID));
		}

		@Test
		void shouldFillGenericBody() throws Exception {
			var content = TestUtils.loadTextFile("jsonTemplates/command/createCommandWithBody.json.tmpl", CommandOrder.CREATE_BESCHEID.name(),
					"{\"bescheidVom\":\"2023-06-26\"}");

			doRequest(content);

			verify(service).createCommand(createCommandCaptor.capture(), anyLong());
			var body = (GenericCommandBody) createCommandCaptor.getValue().getBody();
			assertThat(body).isNotNull().contains(entry("bescheidVom", "2023-06-26"));
		}

		private ResultActions doRequest() throws Exception {
			return doRequest(
					TestUtils.loadTextFile("jsonTemplates/command/createVorgangCommand.json.tmpl", CommandOrder.VORGANG_ANNEHMEN.name()));
		}

		private ResultActions doRequest(String content) throws Exception {
			return mockMvc
					.perform(post(CommandByRelationController.COMMAND_BY_RELATION_PATH, VorgangHeaderTestFactory.ID, CommandTestFactory.RELATION_ID,
							CommandTestFactory.RELATION_VERSION)
									.contentType(MediaType.APPLICATION_JSON)
									.characterEncoding(StandardCharsets.UTF_8.name())
									.content(content))
					.andExpect(status().is2xxSuccessful());
		}
	}

	@DisplayName("Create by vorgang with relation id")
	@Nested
	class TestCreateByVorgang {

		private final CreateCommand command = CommandTestFactory.createCreateCommand();

		@Test
		void shouldCallSerice() {
			controller.createCommand(command);

			verify(service).createCommand(command);
		}
	}
}
