/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.itvsh.kop.common.test.TestUtils;
import de.ozgcloud.alfa.common.ValidationMessageCodes;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.postfach.PostfachMailTestFactory;
import de.ozgcloud.alfa.vorgang.RedirectRequestTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import de.ozgcloud.alfa.vorgang.forwarding.RedirectRequest;

@AutoConfigureMockMvc
@SpringBootTest
public class CommandITCase {

	@Autowired
	private MockMvc mockMvc;
	@MockBean
	private CommandRemoteService commandRemoteService;
	@MockBean
	private VorgangController vorgangController;

	@Nested
	@WithMockUser
	class TestAssignUser {

		@Captor
		private ArgumentCaptor<CreateCommand> commandCaptor;

		@BeforeEach
		void initTests() {
			when(commandRemoteService.createCommand(any())).thenReturn(CommandTestFactory.create());
		}

		@Test
		void shouldExtractUserId() throws Exception {
			createCommand();

			verify(commandRemoteService).createCommand(commandCaptor.capture());
			assertThat(commandCaptor.getValue().getBody()).hasFieldOrPropertyWithValue("assignedTo", UserProfileTestFactory.ID);
		}

		private void createCommand() throws Exception {
			mockMvc.perform(post(buildUrl()).with(csrf()).content(createContent()).contentType(MediaType.APPLICATION_JSON))
					.andExpect(status().isCreated());
		}

		private String createContent() {
			return TestUtils.loadTextFile("jsonTemplates/command/createCommandWithBody.json.tmpl", CommandOrder.ASSIGN_USER.name(),
					TestUtils.loadTextFile("jsonTemplates/command/commandAssignedToBody",
							TestUtils.addQuote("/api/users/" + UserProfileTestFactory.ID.toString())));
		}

		private String buildUrl() {
			return String.format("/api/vorgangs/%s/relations/%s/%d/commands", VorgangHeaderTestFactory.ID, VorgangHeaderTestFactory.ID,
					VorgangHeaderTestFactory.VERSION);
		}
	}

	@DisplayName("Validation")
	@Nested
	class TestValidation {

		@DisplayName("of forwarding request")
		@WithMockUser
		@Nested
		class TestRedirectRequest {

			@DisplayName("with null/empty email")
			@WithMockUser
			@Nested
			class TestOnNullOrEmptyEmail {

				@Test
				void shouldThrowErrorForEmailOnly() throws Exception {
					var requestContent = buildRedirectRequest(RedirectRequestTestFactory.createBuilder()
							.password(ArrayUtils.EMPTY_CHAR_ARRAY)
							.email(null).build());

					doRequest(requestContent).andExpect(status().isUnprocessableEntity())
							.andExpect(jsonPath("$.issues.length()").value(1))
							.andExpect(jsonPath("$.issues.[0].field").value("command.redirectRequest.email"))
							.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_IS_EMPTY));
				}
			}

			@DisplayName("with invalid email and password")
			@WithMockUser
			@Nested
			class TestOnInvalidEmailAndPassword {

				@Test
				void shouldThrowExceptionForEachOne() throws Exception {
					var requestContent = buildRedirectRequest(RedirectRequestTestFactory.createBuilder()
							.password(ArrayUtils.EMPTY_CHAR_ARRAY)
							.email("local@@domain.com")
							.build());

					doRequest(requestContent).andExpect(status().isUnprocessableEntity())
							.andExpect(jsonPath("$.issues.length()").value(2));
				}
			}

			@WithMockUser
			@Nested
			class TestEmail {

				private final String FIELD = "email";

				@Test
				void shouldReturnErrorOnNullEMail() throws Exception {
					var requestContent = buildRedirectRequestWithEmail(null);

					doRequest(requestContent).andExpect(status().isUnprocessableEntity())
							.andExpect(jsonPath("$.issues.length()").value(1))
							.andExpect(jsonPath("$.issues.[0].field").value("command.redirectRequest." + FIELD))
							.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_IS_EMPTY));
				}

				@Test
				void shouldReturnErrorOnInvalidEMail() throws Exception {
					var requestContent = buildRedirectRequestWithEmail("local@@domain.com");

					doRequest(requestContent).andExpect(status().isUnprocessableEntity())
							.andExpect(jsonPath("$.issues.length()").value(1))
							.andExpect(jsonPath("$.issues.[0].field").value("command.redirectRequest." + FIELD))
							.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_INVALID));
				}

				private String buildRedirectRequestWithEmail(String eMail) {
					return buildRedirectRequest(RedirectRequestTestFactory.createBuilder().email(eMail).build());
				}
			}

			@WithMockUser
			@Nested
			class TestPassword {

				private final String FIELD = "password";

				@Test
				void shouldReturnErrorOnToShortPassword() throws Exception {
					var requestContent = buildRedirectRequestWithPassword(RandomStringUtils.randomAlphabetic(7));

					doRequest(requestContent).andExpect(status().isUnprocessableEntity())
							.andExpect(jsonPath("$.issues.length()").value(1))
							.andExpect(jsonPath("$.issues.[0].field").value("command.redirectRequest." + FIELD))
							.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_SIZE))
							.andExpect(jsonPath("$.issues.[0].parameters[0].name").value("min"))
							.andExpect(jsonPath("$.issues.[0].parameters[0].value").value(8))
							.andExpect(jsonPath("$.issues.[0].parameters[1].name").value("max"))
							.andExpect(jsonPath("$.issues.[0].parameters[1].value").value(40));
				}

				@Test
				void shouldReturnErrorOnToLongPassword() throws Exception {
					var requestContent = buildRedirectRequestWithPassword(RandomStringUtils.randomAlphabetic(41));

					doRequest(requestContent).andExpect(status().isUnprocessableEntity())
							.andExpect(jsonPath("$.issues.length()").value(1))
							.andExpect(jsonPath("$.issues.[0].field").value("command.redirectRequest." + FIELD))
							.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_SIZE));
				}

				private String buildRedirectRequestWithPassword(String password) {
					return buildRedirectRequest(RedirectRequestTestFactory.createBuilder().password(password.toCharArray()).build());
				}
			}

			private String buildRedirectRequest(RedirectRequest request) {
				return RedirectRequestTestFactory
						.createRedirectRequestContent(CommandTestFactory.createCreateCommandBuilder().redirectRequest(request).build());
			}
		}

		@WithMockUser
		@Nested
		class TestPostfachMail {

			@BeforeEach
			void mockVorgangController() {
				when(vorgangController.getVorgang(anyString())).thenReturn(VorgangWithEingangTestFactory.create());
			}

			@WithMockUser
			@Nested
			class TestSubject {

				@BeforeEach
				void initMockCommand() {
					when(commandRemoteService.createCommand(any(CreateCommand.class))).thenReturn(CommandTestFactory.create());
				}

				@Test
				void shouldReturnErrorOnNullSubject() throws Exception {
					var requestContent = PostfachMailTestFactory
							.buildSendPostfachMailContent(PostfachMailTestFactory.createBuilder().subject(null).build());

					doRequest(requestContent).andExpect(status().isUnprocessableEntity())
							.andExpect(jsonPath("$.issues.length()").value(1))
							.andExpect(jsonPath("$.issues.[0].field").value("command.body.subject"))
							.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_IS_EMPTY));
				}

				@Test
				void shouldReturnErrorOnTooLongSubject() throws Exception {
					var requestContent = PostfachMailTestFactory
							.buildSendPostfachMailContent(
									PostfachMailTestFactory.createBuilder().subject(RandomStringUtils.randomAlphanumeric(71)).build());

					doRequest(requestContent).andExpect(status().isUnprocessableEntity())
							.andExpect(jsonPath("$.issues.length()").value(1))
							.andExpect(jsonPath("$.issues.[0].field").value("command.body.subject"))
							.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_MAX_SIZE));
				}

				@Test
				void shouldProceedOnValidSubject() throws Exception {
					var requestContent = PostfachMailTestFactory.buildSendPostfachMailContent();

					doRequest(requestContent).andExpect(status().isCreated());
				}
			}

			@WithMockUser
			@Nested
			class TestMailBody {

				@Test
				void shouldReturnErrorOnNull() throws Exception {
					var request = PostfachMailTestFactory
							.buildSendPostfachMailContent(PostfachMailTestFactory.createBuilder().mailBody(null).build());

					doRequest(request).andExpect(status().isUnprocessableEntity())
							.andExpect(jsonPath("$.issues.length()").value(1))
							.andExpect(jsonPath("$.issues.[0].field").value("command.body.mailBody"))
							.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_IS_EMPTY));
				}

				@Test
				void shouldProceedOnValidInput() throws Exception {
					when(commandRemoteService.createCommand(any(CreateCommand.class))).thenReturn(CommandTestFactory.create());

					var content = PostfachMailTestFactory.buildSendPostfachMailContent();

					doRequest(content).andExpect(status().isCreated());
				}
			}
		}

		ResultActions doRequest(String content) throws Exception {
			return mockMvc.perform(post("/api/vorgangs/" + CommandTestFactory.VORGANG_ID + "/relations/" + CommandTestFactory.RELATION_ID + "/"
					+ CommandTestFactory.RELATION_VERSION + "/commands")
							.with(csrf())
							.contentType(MediaType.APPLICATION_JSON)
							.content(content));
		}
	}
}