/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.kommentar;

import static de.ozgcloud.alfa.kommentar.KommentarCommandTestFactory.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.ozgcloud.alfa.common.ValidationMessageCodes;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandRemoteService;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.kommentar.KommentarCommandController.KommentarCommandByVorgangController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

@AutoConfigureMockMvc
@SpringBootTest
@WithMockUser
class KommentarCommandITCase {

	@Autowired
	private MockMvc mockMvc;
	@MockBean
	private KommentarRemoteService remoteService;
	@MockBean
	private CommandRemoteService commandRemoteService;

	@SpyBean
	private CurrentUserService userService;

	@WithMockUser
	@Nested
	class TestCreateCommandByKommentarId {

		@BeforeEach
		void initTest() {
			when(remoteService.getById(any())).thenReturn(KommentarTestFactory.create());
			when(commandRemoteService.createCommand(any())).thenReturn(CommandTestFactory.create());
		}

		@Test
		void shouldCreateCommand() throws Exception {
			doRequestByKommentarId(createValidRequestContent()).andExpect(status().isCreated());

			verify(commandRemoteService).createCommand(any());
		}

		@WithMockUser
		@DisplayName("should return validation error")
		@Nested
		class TestValidation {

			@DisplayName("for null Text")
			@Test
			void createCommandWithInvalidText() throws Exception {
				String content = buildContentWithText(null);

				doRequestByKommentarId(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.issues.length()").value(1))
						.andExpect(jsonPath("$.issues.[0].field").value("kommentar.text"))
						.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_IS_EMPTY));
			}

			@DisplayName("for empty String in Text")
			@Test
			void createcommandWithShortText() throws Exception {
				String content = buildContentWithText(StringUtils.EMPTY);

				doRequestByKommentarId(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.issues.[0].field").value("kommentar.text"));

			}

			@DisplayName("for invalid text should have parameter")
			@Test
			void minMaxParameter() throws Exception {
				String content = buildContentWithText(StringUtils.EMPTY);

				doRequestByKommentarId(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.issues[0].parameters.length()").value(2));
			}

			private String buildContentWithText(String text) {
				return createRequestContent(KommentarCommandTestFactory.createBuilder()
						.order(CommandOrder.CREATE_ATTACHED_ITEM)
						.kommentar(KommentarTestFactory.createBuilder().text(text).build())
						.build());
			}
		}

		private ResultActions doRequestByKommentarId(String content) throws Exception {
			return mockMvc
					.perform(post(KommentarCommandController.KOMMENTAR_COMMANDS, KommentarTestFactory.ID, KommentarTestFactory.VERSION).with(csrf())
							.contentType(MediaType.APPLICATION_JSON)
							.content(content));
		}
	}

	@WithMockUser
	@Nested
	class TestCreateCommandByVorgangId {

		private static final long RELATION_ID_ON_CREATE = -1;

		@BeforeEach
		void initTest() {
			when(remoteService.getById(any())).thenReturn(KommentarTestFactory.create());
			when(commandRemoteService.createCommand(any())).thenReturn(CommandTestFactory.create());

			doReturn(UserProfileTestFactory.ID).when(userService).getUserId();
		}

		@Test
		void shouldCreateCommand() throws Exception {
			doRequestByVorgangId(createValidRequestContent()).andExpect(status().isCreated());

			verify(commandRemoteService).createCommand(any());
		}

		@WithMockUser
		@DisplayName("should return validation error")
		@Nested
		class TestValidation {

			@DisplayName("for null Text")
			@Test
			void createCommandWithInvalidText() throws Exception {
				String content = buildContentWithText(null);

				doRequestByVorgangId(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.issues.length()").value(1))
						.andExpect(jsonPath("$.issues.[0].field").value("kommentar.text"))
						.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_IS_EMPTY));
			}

			@DisplayName("for empty String in Text")
			@Test
			void createcommandWithShortText() throws Exception {
				String content = buildContentWithText(StringUtils.EMPTY);

				doRequestByVorgangId(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.issues.[0].field").value("kommentar.text"));

			}

			@DisplayName("for invalid text should have parameter")
			@Test
			void minMaxParameter() throws Exception {
				String content = buildContentWithText(StringUtils.EMPTY);

				doRequestByVorgangId(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.issues[0].parameters.length()").value(2));
			}

			private String buildContentWithText(String text) {
				return createRequestContent(KommentarCommandTestFactory.createBuilder()
						.order(CommandOrder.CREATE_KOMMENTAR)
						.kommentar(KommentarTestFactory.createBuilder().text(text).build())
						.build());
			}
		}

		private ResultActions doRequestByVorgangId(String content) throws Exception {
			return mockMvc.perform(post(KommentarCommandByVorgangController.KOMMENTAR_COMMANDS_BY_VORGANG, VorgangHeaderTestFactory.ID,
					RELATION_ID_ON_CREATE).with(csrf())
							.contentType(MediaType.APPLICATION_JSON)
							.content(content));
		}
	}
}