package de.ozgcloud.alfa.statistic;

import static de.ozgcloud.alfa.statistic.StatisticRemoteService.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.AlfaProperties;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.vorgang.statistic.GrpcVorgangCountResponse;
import de.ozgcloud.vorgang.statistic.GrpcVorgangStatisticQuery.GroupMethod;
import de.ozgcloud.vorgang.statistic.GrpcVorgangStatisticQuery.Operator;
import de.ozgcloud.vorgang.statistic.GrpcVorgangStatisticResponse;
import de.ozgcloud.vorgang.statistic.GrpcVorgangStatisticResult;
import de.ozgcloud.vorgang.statistic.StatisticServiceGrpc.StatisticServiceBlockingStub;

class StatisticRemoteServiceTest {

	@Spy
	@InjectMocks
	private StatisticRemoteService service;
	@Mock
	private StatisticServiceBlockingStub serviceStub;
	@Mock
	private StatisticMapper mapper;
	@Mock
	private AlfaProperties alfaProperties;

	private static final List<VorgangStatus> STATUSES_TO_COUNT = List.of(VorgangStatus.NEU, VorgangStatus.ANGENOMMEN,
			VorgangStatus.IN_BEARBEITUNG, VorgangStatus.BESCHIEDEN,
			VorgangStatus.ABGESCHLOSSEN, VorgangStatus.VERWORFEN, VorgangStatus.ZU_LOESCHEN);

	@DisplayName("Build count by vorgang status query")
	@Nested
	class TestBuildCountByVorgangStatusQuery {

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "WEITERGELEITET")
		void shouldContainResultName(VorgangStatus vorgangStatus) {
			var query = service.buildCountByVorgangStatusQuery(vorgangStatus);

			assertThat(query.getResultName()).isEqualTo(String.format(COUNT_VORGANG_STATUS_RESULT_NAME_FORMAT, vorgangStatus.name()));
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "WEITERGELEITET")
		void shouldContainPath(VorgangStatus vorgangStatus) {
			var query = service.buildCountByVorgangStatusQuery(vorgangStatus);

			assertThat(query.getPath()).isEqualTo(STATUS_PATH);
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "WEITERGELEITET")
		void shouldContainGroupMethod(VorgangStatus vorgangStatus) {
			var query = service.buildCountByVorgangStatusQuery(vorgangStatus);

			assertThat(query.getGroupMethod()).isEqualTo(GroupMethod.COUNT);
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "WEITERGELEITET")
		void shouldContainOperator(VorgangStatus vorgangStatus) {
			var query = service.buildCountByVorgangStatusQuery(vorgangStatus);

			assertThat(query.getOperator()).isEqualTo(Operator.EQUAL);
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "WEITERGELEITET")
		void shouldContainOperandStringValue(VorgangStatus vorgangStatus) {
			var query = service.buildCountByVorgangStatusQuery(vorgangStatus);

			assertThat(query.getOperandStringValue()).isEqualTo(vorgangStatus.name());
		}
	}

	@Nested
	class TestBuildCountByStatusQueries {

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "WEITERGELEITET")
		void shouldCallBuildCountByVorgangStatusQuery(VorgangStatus vorgangStatus) {
			service.buildCountByStatusQueries(STATUSES_TO_COUNT);

			verify(service).buildCountByVorgangStatusQuery(vorgangStatus);
		}
	}

	@Nested
	class TestBuildExistsWiedervorlageOverdueQuery {

		private final String applicationName = LoremIpsum.getInstance().getFirstName();

		@BeforeEach
		void init() {
			when(alfaProperties.getApplicationName()).thenReturn(applicationName);
		}

		@Test
		void shouldContainResultName() {
			var query = service.buildExistsWiedervorlageOverdueQuery();

			assertThat(query.getResultName()).isEqualTo(StatisticRemoteService.EXISTS_WIEDERVORLAGE_OVERDUE_RESULT_NAME);
		}

		@Test
		void shouldContainPath() {
			var query = service.buildExistsWiedervorlageOverdueQuery();

			assertThat(query.getPath()).isEqualTo(String.format(StatisticRemoteService.WIEDERVORLAGE_NEXT_FRIST_PATH_TEMPLATE, applicationName));
		}

		@Test
		void shouldContainGroupMethod() {
			var query = service.buildExistsWiedervorlageOverdueQuery();

			assertThat(query.getGroupMethod()).isEqualTo(GroupMethod.EXISTS);
		}

		@Test
		void shouldContainOperator() {
			var query = service.buildExistsWiedervorlageOverdueQuery();

			assertThat(query.getOperator()).isEqualTo(Operator.LESS_THEN_OR_EQUAL_TO);
		}

		@Test
		void shouldContainOperandStringValue() {
			var query = service.buildExistsWiedervorlageOverdueQuery();

			assertThat(query.getOperandStringValue()).isEqualTo(StatisticRemoteService.OPERAND_TODAY_DATE);
		}

		@Test
		void shouldCallPropertiesToGetClientName() {
			service.buildExistsWiedervorlageOverdueQuery();

			verify(alfaProperties).getApplicationName();
		}
	}

	@DisplayName("Get vorgaenge statistic")
	@Nested
	class TestGetVorgaengeStatistics {

		private final GrpcVorgangStatisticResponse response = GrpcVorgangStatisticResponseTestFactory.create();

		@BeforeEach
		void beforeEach() {
			when(serviceStub.getVorgangStatistic(any())).thenReturn(response);

			doReturn(GrpcVorgangStatisticQueryTestFactory.create()).when(service).buildExistsWiedervorlageOverdueQuery();
		}

		@Test
		void shouldCallServiceStub() {
			service.getVorgaengeStatistics(Collections.emptyList());

			verify(serviceStub).getVorgangStatistic(any());
		}

		@Test
		void shouldCallBuildCountByStatusQueries() {
			service.getVorgaengeStatistics(STATUSES_TO_COUNT);

			verify(service).buildCountByStatusQueries(STATUSES_TO_COUNT);
		}

		@Test
		void shouldCallBuildExistsWiedervorlageOverdueQuery() {
			service.getVorgaengeStatistics(STATUSES_TO_COUNT);

			verify(service).buildExistsWiedervorlageOverdueQuery();
		}

		@Test
		void shouldCallBuildStatisticResult() {
			service.getVorgaengeStatistics(STATUSES_TO_COUNT);

			verify(service).buildStatisticResult(response);
		}
	}

	@Nested
	class TestBuildStatisticResult {

		private final String RESULT_NAME = "countByStatus";
		private final int RESULT_INT = 42;
		private GrpcVorgangStatisticResponse grpcResponse;
		private GrpcVorgangStatisticResult grpcResult;

		@BeforeEach
		void beforeEach() {
			grpcResult = GrpcVorgangStatisticResult.newBuilder()
					.setName(RESULT_NAME)
					.setResultIntValue(RESULT_INT)
					.build();
			grpcResponse = GrpcVorgangStatisticResponse.newBuilder()
					.addResult(grpcResult)
					.build();
			when(mapper.toResult(grpcResult)).thenReturn(StatisticResult.builder().name(RESULT_NAME).intValue(RESULT_INT).build());
		}

		@Test
		void shouldCallMapper() {
			service.buildStatisticResult(grpcResponse);

			verify(mapper).toResult(grpcResult);
		}

		@Test
		void shouldBuildResult() {
			var result = service.buildStatisticResult(grpcResponse);

			assertThat(result)
					.hasSize(1)
					.hasEntrySatisfying(RESULT_NAME, statisticResult -> assertThat(statisticResult.getIntValue()).isEqualTo(RESULT_INT));
		}
	}

	// TODO sebo: später löschen
	@DisplayName("Get statistic")
	@Nested
	class TestGetStatistic {

		private String countByPath = "DummyCountPath";
		private List<String> countByPathList = List.of(countByPath);
		private final GrpcVorgangCountResponse response = GrpcVorgangCountTestFactory.createResponse();

		@DisplayName("prozess flow")
		@Nested
		class TestProzessFlow {

			@BeforeEach
			void mockServiceStub() {
				when(serviceStub.countVorgang(any())).thenReturn(response);
			}

			@Test
			void shouldCallStub() {
				service.countVorgang(countByPathList);

				verify(serviceStub).countVorgang(any());
			}

			@Test
			void shouldBuildRequest() {
				service.countVorgang(countByPathList);

				verify(service).buildVorgangCountRequest(countByPathList);
			}

			@Test
			void shouldMapCountVorgangResponse() {
				service.countVorgang(countByPathList);

				verify(mapper).fromCountVorgangResponse(response);
			}
		}

		@DisplayName("build vorgang count request")
		@Nested
		class TestBuildVorgangCountRequest {

			@Test
			void shouldHaveCountByPath() {
				var request = service.buildVorgangCountRequest(countByPathList);

				assertThat(request.getCountByPathList()).hasSize(1).containsExactly(countByPath);
			}
		}
	}

}