/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import static de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory.*;
import static org.assertj.core.api.Assertions.*;

import java.time.format.DateTimeFormatter;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserIdMapper;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.alfa.wiedervorlage.WiedervorlageTestFactory;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttribute;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttributeValue;

class VorgangHeaderMapperTest {

	@Spy
	@InjectMocks
	private VorgangHeaderMapper mapper = Mappers.getMapper(VorgangHeaderMapper.class);

	@Spy
	private UserIdMapper userIdMapper = Mappers.getMapper(UserIdMapper.class);

	@Nested
	class TestToVorgangHeader {

		@Test
		void shouldMapCreatedAt() {
			var header = callMapper();

			assertThat(header.getCreatedAt()).isEqualTo(CREATED_AT);
		}

		@Test
		void shouldMapNummer() {
			var header = callMapper();

			assertThat(header.getNummer()).isEqualTo(NUMMER);
		}

		@Test
		void shouldMapAktenzeichen() {
			var header = callMapper();

			assertThat(header.getAktenzeichen()).isEqualTo(AKTENZEICHEN);
		}

		@Test
		void shouldMapStatus() {
			var header = callMapper();

			assertThat(header.getStatus()).isEqualTo(VorgangStatus.NEU);
		}

		@Test
		void shouldMapAssignedTo() {
			var header = callMapper();

			assertThat(header.getAssignedTo()).isEqualTo(UserId.from(ASSIGNED_TO));
		}

		@Nested
		class NextFrist {

			@Test
			void shouldSetEmptyOptionalIfNotExist() {
				var vorgangHeaderWithEmptyClientAttributes = GrpcVorgangHeaderTestFactory.createBuilder().clearClientAttributes().build();

				var header = mapper.toVorgangHeader(vorgangHeaderWithEmptyClientAttributes);

				assertThat(header.getNextFrist()).isNull();
			}

			@Test
			void shouldSetIfExists() {
				var nextWiedervorlageFristClientAttribute = GrpcClientAttribute.newBuilder()
						.setAttributeName(VorgangHeaderMapper.WIEDERVORLAGE_NEXT_FRIST_ATTRIBUTE_NAME)
						.setValue(GrpcClientAttributeValue.newBuilder()
								.setStringValue(WiedervorlageTestFactory.FRIST.format(DateTimeFormatter.ISO_DATE)).build())
						.build();
				var vorgangHeaderWithNextWiedervorlageFristAsClientAttribute = GrpcVorgangHeaderTestFactory.createBuilder()
						.addClientAttributes(nextWiedervorlageFristClientAttribute).build();

				var header = mapper.toVorgangHeader(vorgangHeaderWithNextWiedervorlageFristAsClientAttribute);

				assertThat(header.getNextFrist()).isEqualTo(WiedervorlageTestFactory.FRIST);
			}
		}

		@Nested
		class HasPostfachNachricht {
			@Test
			void shouldSetFalseIfNotExists() {
				var vorgangHeader = GrpcVorgangHeaderTestFactory.createBuilder().clearClientAttributes().build();

				var mapped = mapper.toVorgangHeader(vorgangHeader);

				assertThat(mapped.isHasPostfachNachricht()).isFalse();
			}

			@Test
			void shouldBeTrue() {
				var clientAttribute = GrpcClientAttributeTestFactory.createWith("hasPostfachNachricht", true);
				var vorgangHeader = GrpcVorgangHeaderTestFactory.createBuilder().addClientAttributes(clientAttribute).build();

				var mapped = mapper.toVorgangHeader(vorgangHeader);

				assertThat(mapped.isHasPostfachNachricht()).isTrue();
			}
		}

		@Nested
		class HasNewPostfachNachricht {
			@Test
			void shouldSetFalseIfNotExists() {
				var vorgangHeader = GrpcVorgangHeaderTestFactory.createBuilder().clearClientAttributes().build();

				var mapped = mapper.toVorgangHeader(vorgangHeader);

				assertThat(mapped.isHasNewPostfachNachricht()).isFalse();
			}

			@Test
			void shouldBeTrue() {
				var clientAttribute = GrpcClientAttributeTestFactory.createWith("hasNewPostfachNachricht", true);
				var vorgangHeader = GrpcVorgangHeaderTestFactory.createBuilder().addClientAttributes(clientAttribute).build();

				var mapped = mapper.toVorgangHeader(vorgangHeader);

				assertThat(mapped.isHasNewPostfachNachricht()).isTrue();
			}
		}

		private VorgangHeader callMapper() {
			return mapper.toVorgangHeader(GrpcVorgangHeaderTestFactory.create());
		}
	}
}
