package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.TestUtils;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.vorgang.EingangTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import de.xoev.xdomea.AllgemeineMetadatenType;
import de.xoev.xdomea.DokumentType;
import de.xoev.xdomea.IdentifikationObjektType;
import de.xoev.xdomea.VersionType;

class DokumentTypeCreatorTest {

	@Spy
	@InjectMocks
	private DokumentTypeCreator creator;

	@Mock
	private ExportFileService exportFileService;

	@Mock
	private UUIDConverter uuidConverter;

	@Mock
	private VersionTypeCreator versionTypeCreator;

	@Nested
	class TestCreate {

		@Nested
		class WhenAllVorgangDataAvailable {
			private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
			private final OzgFile ozgFile = OzgFileTestFactory.create();

			@BeforeEach
			void init() {
				when(exportFileService.getAllPdfs(EingangTestFactory.ID)).thenReturn(Stream.of(ozgFile));
			}

			@Test
			void shouldLoadlAllPdfs() {
				creator.create(vorgang);

				verify(exportFileService).getAllPdfs(EingangTestFactory.ID);
			}

			@Test
			void shouldCreateDokumentType() {
				creator.create(vorgang);

				verify(creator).createDokumentType(1, vorgang, ozgFile);
			}
		}

		@Nested
		class WhenEingangIsNull {

			@Test
			void shouldNotLoadAnyPdf() {
				creator.create(VorgangWithEingang.builder().eingang(null).build());

				verify(exportFileService, never()).getAllPdfs(EingangTestFactory.ID);
			}

			@Test
			void shouldReturnEmptyStreamIfEingangNull() {
				var result = creator.create(VorgangWithEingang.builder().eingang(null).build());

				assertThat(result).isEmpty();
			}
		}

	}

	@Nested
	class CreateDokumentType {

		private final IdentifikationObjektType identifikationObjekt = IdentifikationObjektTypeTestFactory.create();
		private final AllgemeineMetadatenType allgemeineMetadaten = AllgemeineMetadatenTypeTestFactory.create();
		private final OzgFile ozgFile = OzgFileTestFactory.create();
		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
		private final VersionType version = VersionTypeTestFactory.create();
		private final int DOKUMENT_ORDINAL_NUMBER = 1;

		@BeforeEach
		void init() {
			doReturn(identifikationObjekt).when(creator).createIdentifikation(DOKUMENT_ORDINAL_NUMBER, OzgFileTestFactory.ID.toString());
			doReturn(allgemeineMetadaten).when(creator).createAllgemeineMetadaten(ozgFile);

			when(versionTypeCreator.create(vorgang, ozgFile)).thenReturn(version);
		}

		@Test
		void shouldCreateIdentifikation() {
			create();

			verify(creator).createIdentifikation(DOKUMENT_ORDINAL_NUMBER, OzgFileTestFactory.ID.toString());
		}

		@Test
		void shouldHaveIdentifikation() {
			var dokument = create();

			assertThat(dokument.getIdentifikation()).isEqualTo(identifikationObjekt);
		}

		@Test
		void shouldCreateAllgemeineMetadaten() {
			create();

			verify(creator).createAllgemeineMetadaten(ozgFile);
		}

		@Test
		void shouldHaveAllgemeineMetadaten() {
			var dokument = create();

			assertThat(dokument.getAllgemeineMetadaten()).isEqualTo(allgemeineMetadaten);
		}

		@Test
		void shouldCreateVersion() {
			create();

			verify(versionTypeCreator).create(vorgang, ozgFile);
		}

		@Test
		void shouldHaveVersion() {
			var dokument = create();

			assertThat(dokument.getVersion()).containsExactly(version);
		}

		private DokumentType create() {
			return creator.createDokumentType(DOKUMENT_ORDINAL_NUMBER, vorgang, ozgFile);
		}
	}

	@Nested
	class TestCreateIdentifikation {

		private final String FILE_ID = "64a820d36285172ac02826d0";
		private final String FILE_UUID = "64a820d3-6285-172a-c028-0000000026d0";
		private final int DOKUMENT_ORDINAL_NUMBER = 1;

		@BeforeEach
		void init() {
			when(uuidConverter.fromObjectId(FILE_ID)).thenReturn(FILE_UUID);
		}

		@Test
		void shouldConvertObjectId() {
			create();

			verify(uuidConverter).fromObjectId(FILE_ID);
		}

		@Test
		void shouldHaveId() {
			var identifikation = create();

			assertThat(identifikation.getID()).matches(TestUtils.UUID_REGEX).isEqualTo(FILE_UUID);
		}

		@Test
		void shouldHaveNummerImUebergeordnetenContainer() {
			var identifikation = create();

			assertThat(identifikation.getNummerImUebergeordnetenContainer()).isEqualTo(DOKUMENT_ORDINAL_NUMBER);
		}

		private IdentifikationObjektType create() {
			return creator.createIdentifikation(DOKUMENT_ORDINAL_NUMBER, FILE_ID);
		}
	}

	@Nested
	class TestCreateAllgemeineMetadaten {

		private final OzgFile ozgFile = OzgFileTestFactory.create();

		@Test
		void shouldHaveBetreff() {
			var allgemeineMetadaten = creator.createAllgemeineMetadaten(ozgFile);

			assertThat(allgemeineMetadaten.getBetreff()).isEqualTo(OzgFileTestFactory.NAME);
		}

		@Test
		void shouldHaveKennzeichen() {
			var allgemeineMetadaten = creator.createAllgemeineMetadaten(ozgFile);

			assertThat(allgemeineMetadaten.getKennzeichen()).isEmpty();
		}

		@Test
		void shouldHaveBemerkung() {
			var allgemeineMetadaten = creator.createAllgemeineMetadaten(ozgFile);

			assertThat(allgemeineMetadaten.getBemerkung()).isEmpty();
		}

		@Test
		void shouldHaveMedium() {
			var allgemeineMetadaten = creator.createAllgemeineMetadaten(ozgFile);

			assertThat(allgemeineMetadaten.getMedium().getCode()).isEqualTo(VorgangTypeCreator.ALLGEMEINE_METADATEN_MEDIUM_CODE);
		}

	}

}