package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

import de.ozgcloud.alfa.vorgang.AntragstellerTestFactory;
import de.ozgcloud.alfa.vorgang.EingangTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import de.xoev.xdomea.DatentypCodeType;
import de.xoev.xdomea.FeldType;

class FeldGruppeTypeCreatorTest {

	@Spy
	private FeldGruppeTypeCreator creator;

	@Nested
	class TestCreate {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		@Test
		void shouldHaveName() {
			var created = creator.create(vorgang);

			assertThat(created.getName()).isEqualTo("FeldgruppeOZGCloudBasis");
		}

		@Test
		void shouldHaveBeschreibung() {
			var created = creator.create(vorgang);

			assertThat(created.getBeschreibung()).isEqualTo("Feldgruppe für OZGCloud Basis");
		}

		@Test
		void shouldCreateFeldType() {
			creator.create(vorgang);

			verify(creator).createFeldType(vorgang);
		}
	}

	@Nested
	class TestCreateFeldType {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		@DisplayName("LeikaId")
		@Nested
		class TestLeikaId {

			@Test
			void shouldHaveName() {
				var created = create();

				assertThat(created).extracting(FeldType::getName).isEqualTo("LeikaID");
			}

			@Test
			void shouldHaveBeschreibung() {
				var created = create();

				assertThat(created).extracting(FeldType::getBeschreibung).isEqualTo("ID einer Leistung aus dem OZG-Leistungskatalog");
			}

			@Test
			void shouldHaveWert() {
				var created = create();

				assertThat(created).extracting(FeldType::getWert).isNull();
			}

			@DisplayName("DatentypCode")
			@Nested
			class TestDatentypCode {

				@Test
				void shouldHaveCode() {
					var created = create();

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getCode)
							.isEqualTo(DatentypCode.STRING.getCode());
				}

				@Test
				void shouldHaveListURI() {
					var created = create();

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getListURI)
							.isEqualTo("urn:xoev-de:xdomea:codeliste:datentyp");
				}

				@Test
				void shouldHaveListVersionID() {
					var created = create();

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getListVersionID).isEqualTo("1.1");
				}

			}

			private FeldType create() {
				return creator.createFeldType(vorgang).stream()
						.filter(feldType -> feldType.getName().equals(ExportFelder.LEIKA_ID.getName()))
						.toList().get(0);
			}
		}

		@DisplayName("Datum Antragseingang")
		@Nested
		class TestDatumAntragseingang {

			@Test
			void shouldHaveName() {
				var created = create();

				assertThat(created).extracting(FeldType::getName).isEqualTo("DatumAntragseingang");
			}

			@Test
			void shouldHaveBeschreibung() {
				var created = create();

				assertThat(created).extracting(FeldType::getBeschreibung).isEqualTo("Das Datum des Antragseingangs");
			}

			@Test
			void shouldHaveWert() {
				var created = create();

				assertThat(created).extracting(FeldType::getWert).isEqualTo(VorgangHeaderTestFactory.CREATED_AT_STR);
			}

			@DisplayName("DatentypCode")
			@Nested
			class TestDatentypCode {

				@Test
				void shouldHaveCode() {
					var created = create();

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getCode)
							.isEqualTo(DatentypCode.DATE.getCode());
				}

				@Test
				void shouldHaveListURI() {
					var created = create();

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getListURI)
							.isEqualTo("urn:xoev-de:xdomea:codeliste:datentyp");
				}

				@Test
				void shouldHaveListVersionID() {
					var created = create();

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getListVersionID).isEqualTo("1.1");
				}

			}

			private FeldType create() {
				return creator.createFeldType(vorgang).stream()
						.filter(feldType -> feldType.getName().equals(ExportFelder.DATUM_ANTRAGSEINGANG.getName()))
						.toList().get(0);
			}
		}

		@DisplayName("Name")
		@Nested
		class TestName {

			@Test
			void shouldHaveName() {
				var created = create(vorgang);

				assertThat(created).extracting(FeldType::getName).isEqualTo("Name");
			}

			@Test
			void shouldNotHaveBeschreibung() {
				var created = create(vorgang);

				assertThat(created).extracting(FeldType::getBeschreibung).isNull();
			}

			@Test
			void shouldHaveWert() {
				var created = create(vorgang);

				assertThat(created).extracting(FeldType::getWert).isEqualTo(AntragstellerTestFactory.NACHNAME);
			}

			@Test
			void shouldBeEmptyStringForNullEingang() {
				var created = create(VorgangWithEingangTestFactory.createBuilder().eingang(null).build());

				assertThat(created.getWert()).isEmpty();
			}

			@Test
			void shouldBeEmptyStringForNullAntragsteller() {
				var created = create(
						VorgangWithEingangTestFactory.createBuilder().eingang(EingangTestFactory.createBuilder().antragsteller(null).build())
								.build());

				assertThat(created.getWert()).isEmpty();
			}

			@Test
			void shouldBeEmptyStringForNullNachname() {
				var created = create(
						VorgangWithEingangTestFactory.createBuilder().eingang(
										EingangTestFactory.createBuilder().antragsteller(AntragstellerTestFactory.createBuilder().nachname(null).build())
												.build())
								.build());

				assertThat(created.getWert()).isEmpty();
			}

			@DisplayName("DatentypCode")
			@Nested
			class TestDatentypCode {

				@Test
				void shouldHaveCode() {
					var created = create(vorgang);

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getCode)
							.isEqualTo(DatentypCode.STRING.getCode());
				}

				@Test
				void shouldHaveListURI() {
					var created = create(vorgang);

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getListURI)
							.isEqualTo("urn:xoev-de:xdomea:codeliste:datentyp");
				}

				@Test
				void shouldHaveListVersionID() {
					var created = create(vorgang);

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getListVersionID).isEqualTo("1.1");
				}

			}

			private FeldType create(VorgangWithEingang vorgang) {
				return creator.createFeldType(vorgang).stream()
						.filter(feldType -> feldType.getName().equals(ExportFelder.NAME.getName()))
						.toList().get(0);
			}
		}

		@DisplayName("Vorname")
		@Nested
		class TestVorname {

			@Test
			void shouldHaveName() {
				var created = create(vorgang);

				assertThat(created).extracting(FeldType::getName).isEqualTo("Vorname");
			}

			@Test
			void shouldHaveNotHaveBeschreibung() {
				var created = create(vorgang);

				assertThat(created).extracting(FeldType::getBeschreibung).isNull();
			}

			@Test
			void shouldHaveWert() {
				var created = create(vorgang);

				assertThat(created).extracting(FeldType::getWert).isEqualTo(AntragstellerTestFactory.VORNAME);
			}

			@Test
			void shouldBeEmptyStringForNullEingang() {
				var created = create(VorgangWithEingangTestFactory.createBuilder().eingang(null).build());

				assertThat(created.getWert()).isEmpty();
			}

			@Test
			void shouldBeEmptyStringForNullAntragsteller() {
				var created = create(
						VorgangWithEingangTestFactory.createBuilder().eingang(EingangTestFactory.createBuilder().antragsteller(null).build())
								.build());

				assertThat(created.getWert()).isEmpty();
			}

			@Test
			void shouldBeEmptyStringForNullVorname() {
				var created = create(
						VorgangWithEingangTestFactory.createBuilder().eingang(
										EingangTestFactory.createBuilder().antragsteller(AntragstellerTestFactory.createBuilder().vorname(null).build())
												.build())
								.build());

				assertThat(created.getWert()).isEmpty();
			}

			@DisplayName("DatentypCode")
			@Nested
			class TestDatentypCode {

				@Test
				void shouldHaveCode() {
					var created = create(vorgang);

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getCode)
							.isEqualTo(DatentypCode.STRING.getCode());
				}

				@Test
				void shouldHaveListURI() {
					var created = create(vorgang);

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getListURI)
							.isEqualTo("urn:xoev-de:xdomea:codeliste:datentyp");
				}

				@Test
				void shouldHaveListVersionID() {
					var created = create(vorgang);

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getListVersionID).isEqualTo("1.1");
				}

			}

			private FeldType create(VorgangWithEingang vorgang) {
				return creator.createFeldType(vorgang).stream()
						.filter(feldType -> feldType.getName().equals(ExportFelder.VORNAME.getName()))
						.toList().get(0);
			}
		}

		@DisplayName("Geburtsdatum")
		@Nested
		class TestGeburtsdatum {

			@Test
			void shouldHaveName() {
				var created = create(vorgang);

				assertThat(created).extracting(FeldType::getName).isEqualTo("Geburtsdatum");
			}

			@Test
			void shouldHaveNotHaveBeschreibung() {
				var created = create(vorgang);

				assertThat(created).extracting(FeldType::getBeschreibung).isNull();
			}

			@Test
			void shouldHaveWert() {
				var created = create(vorgang);

				assertThat(created).extracting(FeldType::getWert).isEqualTo(AntragstellerTestFactory.GEBURTSDATUM_STR);
			}

			@Test
			void shouldBeEmptyStringForNullEingang() {
				var created = create(VorgangWithEingangTestFactory.createBuilder().eingang(null).build());

				assertThat(created.getWert()).isEmpty();
			}

			@Test
			void shouldBeEmptyStringForNullAntragsteller() {
				var created = create(
						VorgangWithEingangTestFactory.createBuilder().eingang(EingangTestFactory.createBuilder().antragsteller(null).build())
								.build());

				assertThat(created.getWert()).isEmpty();
			}

			@Test
			void shouldBeEmptyStringForNullGeburtsdatum() {
				var created = create(
						VorgangWithEingangTestFactory.createBuilder().eingang(
										EingangTestFactory.createBuilder().antragsteller(AntragstellerTestFactory.createBuilder().geburtsdatum(null).build())
												.build())
								.build());

				assertThat(created.getWert()).isEmpty();
			}

			@DisplayName("DatentypCode")
			@Nested
			class TestDatentypCode {

				@Test
				void shouldHaveCode() {
					var created = create(vorgang);

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getCode)
							.isEqualTo(DatentypCode.DATE.getCode());
				}

				@Test
				void shouldHaveListURI() {
					var created = create(vorgang);

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getListURI)
							.isEqualTo("urn:xoev-de:xdomea:codeliste:datentyp");
				}

				@Test
				void shouldHaveListVersionID() {
					var created = create(vorgang);

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getListVersionID).isEqualTo("1.1");
				}

			}

			private FeldType create(VorgangWithEingang vorgang) {
				return creator.createFeldType(vorgang).stream()
						.filter(feldType -> feldType.getName().equals(ExportFelder.GEBURTSDATUM.getName()))
						.toList().get(0);
			}
		}

		@DisplayName("Plz")
		@Nested
		class TestPlz {

			@Test
			void shouldHaveName() {
				var created = create(vorgang);

				assertThat(created).extracting(FeldType::getName).isEqualTo("PLZAntragsteller");
			}

			@Test
			void shouldHaveNotHaveBeschreibung() {
				var created = create(vorgang);

				assertThat(created).extracting(FeldType::getBeschreibung).isNull();
			}

			@Test
			void shouldHaveWert() {
				var created = create(vorgang);

				assertThat(created).extracting(FeldType::getWert).isEqualTo(AntragstellerTestFactory.PLZ);
			}

			@Test
			void shouldBeEmptyStringForNullEingang() {
				var created = create(VorgangWithEingangTestFactory.createBuilder().eingang(null).build());

				assertThat(created.getWert()).isEmpty();
			}

			@Test
			void shouldBeEmptyStringForNullAntragsteller() {
				var created = create(
						VorgangWithEingangTestFactory.createBuilder().eingang(EingangTestFactory.createBuilder().antragsteller(null).build())
								.build());

				assertThat(created.getWert()).isEmpty();
			}

			@Test
			void shouldBeEmptyStringForNullPlz() {
				var created = create(
						VorgangWithEingangTestFactory.createBuilder().eingang(
										EingangTestFactory.createBuilder().antragsteller(AntragstellerTestFactory.createBuilder().plz(null).build())
												.build())
								.build());

				assertThat(created.getWert()).isEmpty();
			}

			@DisplayName("DatentypCode")
			@Nested
			class TestDatentypCode {

				@Test
				void shouldHaveCode() {
					var created = create(vorgang);

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getCode)
							.isEqualTo(DatentypCode.STRING.getCode());
				}

				@Test
				void shouldHaveListURI() {
					var created = create(vorgang);

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getListURI)
							.isEqualTo("urn:xoev-de:xdomea:codeliste:datentyp");
				}

				@Test
				void shouldHaveListVersionID() {
					var created = create(vorgang);

					assertThat(created).extracting(FeldType::getDatentyp).extracting(DatentypCodeType::getListVersionID).isEqualTo("1.1");
				}

			}

			private FeldType create(VorgangWithEingang vorgang) {
				return creator.createFeldType(vorgang).stream()
						.filter(feldType -> feldType.getName().equals(ExportFelder.PLZ.getName()))
						.toList().get(0);
			}
		}
	}
}