package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import javax.xml.datatype.XMLGregorianCalendar;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.vorgang.Antragsteller;
import de.ozgcloud.alfa.vorgang.AntragstellerTestFactory;
import de.ozgcloud.alfa.vorgang.EingangTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import de.xoev.xdomea.GeburtType;
import de.xoev.xdomea.KontaktType;
import de.xoev.xdomea.NameNatuerlichePersonType;

class KontaktTypeCreatorTest {

	@Spy
	@InjectMocks
	private KontaktTypeCreator creator;

	@Mock
	private NameNatuerlichePersonTypeCreator nameNatuerlichePersonTypeCreator;

	@Mock
	private DateConverter dateConverter;

	@Nested
	class TestCreate {

		@Nested
		class TestWithAntragsteller {

			private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
			private final KontaktType kontaktType = KontaktTypeTestFactory.create();

			@BeforeEach
			void setUp() {
				doReturn(Optional.of(vorgang.getEingang().getAntragsteller())).when(creator).getAntragstellerIfHasRequiredData(vorgang);
				doReturn(kontaktType).when(creator).toKontaktType(vorgang.getEingang().getAntragsteller());
			}

			@Test
			void shouldGetAntragsteller() {
				callCreator(vorgang);

				verify(creator).getAntragstellerIfHasRequiredData(vorgang);
			}

			@Test
			void shouldMapToKontaktType() {
				callCreator(vorgang);

				verify(creator).toKontaktType(vorgang.getEingang().getAntragsteller());
			}

			@Test
			void shouldReturnKontakt() {
				var kontakt = callCreator(vorgang);

				assertThat(kontakt).get().isEqualTo(kontaktType);
			}
		}

		@Nested
		class TestWithoutAntragsteller {

			private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.createBuilder()
					.eingang(EingangTestFactory.createBuilder().antragsteller(null).build())
					.build();

			@BeforeEach
			void setUp() {
				doReturn(Optional.empty()).when(creator).getAntragstellerIfHasRequiredData(vorgang);
			}

			@Test
			void shouldBeEmpty() {
				var kontakt = callCreator(vorgang);

				assertThat(kontakt).isEmpty();
			}
		}

		private Optional<KontaktType> callCreator(VorgangWithEingang vorgang) {
			return creator.create(vorgang);
		}
	}

	@Nested
	class TestToKontaktType {

		@Mock
		private XMLGregorianCalendar geburtsdatum;

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
		private final NameNatuerlichePersonType nameNatuerlichePerson = NameNatuerlichePersonTypeTestFactory.create();
		private final GeburtType geburtType = GeburtTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			when(nameNatuerlichePersonTypeCreator.create(vorgang.getEingang().getAntragsteller())).thenReturn(nameNatuerlichePerson);
			when(dateConverter.convertGermanFormatToISO(AntragstellerTestFactory.GEBURTSDATUM_STR)).thenReturn(Optional.of(geburtsdatum));
			doReturn(geburtType).when(creator).createGeburtType(Optional.of(geburtsdatum));
		}

		@Test
		void shouldCreateNameNatuerlichePerson() {
			callCreator(vorgang);

			verify(nameNatuerlichePersonTypeCreator).create(vorgang.getEingang().getAntragsteller());
		}

		@Test
		void shouldHaveNameNatuerlichePerson() {
			var kontakt = callCreator(vorgang);

			assertThat(kontakt.getName()).isEqualTo(nameNatuerlichePerson);
		}

		@Test
		void shouldCallDateConverter() {
			callCreator(vorgang);

			verify(dateConverter).convertGermanFormatToISO(AntragstellerTestFactory.GEBURTSDATUM_STR);
		}

		@Test
		void shouldCreateGeburtType() {
			callCreator(vorgang);

			verify(creator).createGeburtType(Optional.of(geburtsdatum));
		}

		@Test
		void shouldHaveGeburtType() {
			var kontakt = callCreator(vorgang);

			assertThat(kontakt.getGeburt()).isEqualTo(geburtType);
		}

		private KontaktType callCreator(VorgangWithEingang vorgang) {
			return creator.toKontaktType(vorgang.getEingang().getAntragsteller());
		}
	}

	@Nested
	class TestCreateGeburtType {

		@Mock
		private XMLGregorianCalendar geburtsdatum;

		@Test
		void shouldCreate() {
			var geburtType = callCreator(Optional.of(geburtsdatum));

			assertThat(geburtType.getDatum()).isEqualTo(geburtsdatum);
		}

		@Test
		void shouldCreateEmptyGeburt() {
			var geburtType = callCreator(Optional.empty());

			assertThat(geburtType.getDatum()).isNull();
		}

		private GeburtType callCreator(Optional<XMLGregorianCalendar> geburtsdatum) {
			return creator.createGeburtType(geburtsdatum);
		}
	}

	@Nested
	class TestGetAntragstellerIfHasRequiredData {

		@Mock
		private XMLGregorianCalendar geburtsdatum;

		@Test
		void shouldBeEmptyIfAntragstellerNull() {
			var antragsteller = callCreator(buildVorgang(null));

			assertThat(antragsteller).isEmpty();
		}

		@Test
		void shouldBeEmptyIfNoneOfRequiredDataExists() {
			when(dateConverter.convertGermanFormatToISO(null)).thenReturn(Optional.empty());

			var antragsteller = callCreator(buildVorgang(Antragsteller.builder().build()));

			assertThat(antragsteller).isEmpty();
		}

		@Test
		void shouldNotBeEmptyIfHasVorname() {
			var antragsteller = callCreator(buildVorgang(Antragsteller.builder().vorname(AntragstellerTestFactory.VORNAME).build()));

			assertThat(antragsteller).isNotEmpty();
		}

		@Test
		void shouldNotBeEmptyIfHasNachname() {
			var antragsteller = callCreator(buildVorgang(Antragsteller.builder().nachname(AntragstellerTestFactory.NACHNAME).build()));

			assertThat(antragsteller).isNotEmpty();
		}

		@Test
		void shouldNotBeEmptyIfHasAnrede() {
			var antragsteller = callCreator(buildVorgang(Antragsteller.builder().anrede(AntragstellerTestFactory.ANREDE).build()));

			assertThat(antragsteller).isNotEmpty();
		}

		@Test
		void shouldNotBeEmptyIfHasGeburtsdatum() {
			when(dateConverter.convertGermanFormatToISO(AntragstellerTestFactory.GEBURTSDATUM_STR)).thenReturn(Optional.of(geburtsdatum));

			var antragsteller = callCreator(buildVorgang(Antragsteller.builder().geburtsdatum(AntragstellerTestFactory.GEBURTSDATUM_STR).build()));

			assertThat(antragsteller).isNotEmpty();
		}

		private Optional<Antragsteller> callCreator(VorgangWithEingang vorgang) {
			return creator.getAntragstellerIfHasRequiredData(vorgang);
		}

		private VorgangWithEingang buildVorgang(Antragsteller antragsteller) {
			return VorgangWithEingangTestFactory.createBuilder().eingang(EingangTestFactory.createBuilder().antragsteller(antragsteller).build())
					.build();
		}
	}

}